/*-------------------------------------------------------------------------
/* UseVote Configuration Reader (c)1994 Ron Dippold All Rights Reserved
/* UseVote Version 3.0
/*
/* This is based on the original configuration code by Ian Kluft.
/* Sorry about the name changes, Ian, I hate long variable names...
/*
/* This module reads USEVOTE.CFG and grabs all the configuration info
/* for a particular vote.  It also contains some commonly used routines.
/*------------------------------------------------------------------------*/

#define UVCONFIG                  /* for usevote.h */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "usevote.h"

#define CLSIZE 300
static char line[CLSIZE];          /* input line */

/* single item numeric configuration items */
int u_votetype,                   /* CFV, STV, or STVCFV */
    u_stv,                        /* STV type of vote */
    u_groups,                     /* number of groups */
    u_needname,                   /* need voter name? */
    u_wantname,                   /* want voter name? */
    u_explicitvote,               /* explicit "vote"? */
    u_maxvotes,                   /* maximum number of votes */
    u_memsort,                    /* do sort in memory? */
    u_dovoteack,                  /* do vote acknowledgement? */
    u_rulesmax,                   /* maximum number of rules */
    u_moderated[MAXGROUPS];        /* group moderated or not */

/* character items */
char *u_votename,                 /* name of vote */
     *u_voteacksub,               /* acknowledgement subject */
     *u_votemark,                 /* ballot marker */
     *u_nametext,                 /* name entry text */
     *u_acktxtfile,               /* file for acknowledgement text */
     *u_cmdfile,                  /* file for domail script */
     *u_rulesfile,                /* file for rules */
     *groupstxt[MAXGROUPS];        /* group names */


void tostring( /* char **, char *,  char * */ );

/* ------------------------------------------------------------------
/* Check if a character is white space
/* ------------------------------------------------------------------
*/
int space( c )
char c;                           /* character to check */
{
  if( c == ' ' || c == '\t' || c == '\n' ) {
    return( 1 );
  } else {
    return( 0 );
  }
}

/* ------------------------------------------------------------------
/* Convert lower case to upper case
/* ------------------------------------------------------------------
*/
char upcase( c )
char c;                           /* character to convert */
{
  if( c>='a' && c<='z' ) return( c-('a'-'A'));
  return( c );
}


/* ------------------------------------------------------------------
/* is letter - check for a-z,  A-Z
/* ------------------------------------------------------------------
*/
int isletter( letter )
char letter;                      /* letter to check */
{
  if( ( letter >='a' && letter<='z' ) ||
      ( letter >='A' && letter<='Z' ) ) {
    return( 1 );
  } else {
    return( 0 );
  }
}

/* ----------------------------------------------------------------------
/*  Compare strings, ignoring case, for a specific length.  Apparently
/*  strnicmp() isn't standard.
/* ----------------------------------------------------------------------
*/
int strlook( str1, str2, len )
char *str1;                       /* first string */
char *str2;                       /* second string */
int len;                          /* length to compare for */
{
  while ( len-- )
    if( ( *str1++ | 0x20 ) != ( *str2++ | 0x20 ) )
      return( 1 );
  return(0);
}

/* ----------------------------------------------------------------------
/*  Strip all whitespace from ends of string
/*  This includes \n
/* ----------------------------------------------------------------------
*/
void stripstr( str )
char *str;                        /* string to process */
{
  int i, j;

  for( i=0; space( str[i] ); i++ );   /* skip spaces */
  for( j=0; str[i] && str[i]!='\n'; str[j++]=str[i++] );
  do {
    str[j--] = 0;                 /* zero terminate */
  } while ( j>=0 && space( str[j] ) );
}

/* ----------------------------------------------------
/* format name and addr into line
/* ----------------------------------------------------
*/
void format( line, addr, name, altname, rm )
char *line;                       /* line to format into */
char *addr;                       /* address */
char *name;                       /* name */
int altname;                      /* alternate name format */
int rm;                           /* right margin */
{
  int i, j;

  for( i=0; addr[i] && i<rm; i++ ) {
    line[i] = addr[i];
  }
  if( addr[i] ) {                 /* end of line */
    line[i] = 0;
    return;
  }

  if( !name[0] || ( altname && (rm-i)<3 ) ) {      /* no name */
    while( i<rm ) line[i++]=' ';
    line[i]=0;
    return;
  }

  if( altname ) {                 /* alternate name printing */
    line[i++] = ' ';
    line[i++] = '(';
    for( j=0; name[j] && i<rm; i++, j++ ) line[i] = name[j];
    if( i >= rm ) {
      line[i] = 0;
      return;
    }
    line[i++] = ')';
    while( i<rm ) {
      line[i++] = ' ';
    }
    line[i] = 0;
    return;
  } else {                        /* standard name printing */
    j = rm - strlen( name );      /* position for name */
    if( j < i+4 ) j = i+4;
    while( i<j && i<rm ) line[i++]=' ';
    if( i >= rm ) {
      line[i] = 0;
      return;
    }

    for( i=0; name[i] && j<rm; i++, j++ ) line[j] = name[i];
    line[j] = 0;
  }
}


/* ******************************************************************* */

/* ------------------------------------------------------------------
/* Initialize all the variables
/* ------------------------------------------------------------------
*/
static void init_config() {
  int i;

  u_votetype = CFV_T;             /* standard CFV */
  u_stv = FALSE;
  u_groups = 0;                   /* no groups */
  u_needname = FALSE;             /* don't need name */
  u_wantname = FALSE;             /* don't want name */
  u_explicitvote = FALSE;         /* don't need "vote" */
  u_maxvotes = 3000;              /* default to 3000 */
  u_memsort = TRUE;               /* do memory sort */
  u_dovoteack = FALSE;            /* assume undefined */
  u_rulesmax = 10;                /* 10 rules */

  u_votename = "%";
  u_voteacksub = "% Vote Ack";
  u_votemark = NULL;              /* no votemark */
  u_nametext = NULL;              /* no name text */
  u_acktxtfile = "acktext.txt";   /* acknowledgement text */
  u_cmdfile = "domail";           /* default mail file name */
  u_rulesfile= "usevote.rul";     /* rules file name */

  for( i=0; i<MAXGROUPS; i++ ) {
    groupstxt[i] = NULL;          /* no group name yet */
    u_moderated[i] = FALSE;       /* not moderated */
  }
}


/* ------------------------------------------------------------------
/* Take line from config file, save in config variable.
/* Allocates space for config variable.
/* ------------------------------------------------------------------
*/
void tostring( var, line, rtext)
char **var;                       /* var to copy to */
char *line;                       /* to get text from */
char *rtext;                      /* replace % with this text */
{
  int i, j, k;
  static char temp[200];


  for( i = j = 0; line[i] && line[i]!='\n'; i++ ) {
    if( line[i] == '%' && rtext ) { /* substitute */
      for( k=0; rtext[k]; temp[j++] = rtext[k++] );
    } else {
      temp[j++] = line[i];
    }
  }
  temp[j++] = '\0';                  /* terminate */
  *var = (char *)malloc( j * sizeof(char));
  while( --j >= 0 ) (*var)[j]=temp[j];
}


/* ------------------------------------------------------------------
/* Read from the file
/* ------------------------------------------------------------------
*/
static void read_config()
{
  FILE *fin;
  int i, j;

  if ( !( fin = fopen( CFGFILE, UV_OPEN_READ ))) {
    fprintf( stderr, "Can't open %s for input\n", CFGFILE );
    exit( 100 );
  }
  for( ;; ) {                     /* for each line in file */
    if( !fgets( line, CLSIZE, fin ) ) {
      break;                      /* done with file */
    }
    for( i=0; line[i]; i++ ) {
      if( line[i] == '#' || line[i] == '\n' ) { /* end of line */
        line[i++] = '\n';
        line[i] = '\0';
        break;
      }
    }
    stripstr( line );             /* strip spaces */
    for( i=0; line[i] && !space(line[i]); i++ ); /* end of first word */
    if( i == 0 ) continue;        /* nothing on line */
    for( j=i; line[j] && space(line[j]); j++ ); /* parameter */

    if( strlook( line, "VOTETYPE", 8 ) == 0 ) {
      u_votetype = atoi( &line[j] );
      if( !line[j] || u_votetype<CFV_T || u_votetype>STVCFV_T ) {
        fprintf( stderr, "  %s: VOTETYPE needs argument of 0, 1 or 2\n", CFGFILE );
        fclose( fin );  exit( 101 );
      }
      u_stv = ( u_votetype == STV_T || u_votetype == STVCFV_T );
    } else if( strlook( line, "GROUP", 5 ) == 0 ) {
      if( u_groups == MAXGROUPS ) {
        fprintf( stderr, "  %s: Maximum of %d GROUPs\n", CFGFILE, MAXGROUPS );
        fclose( fin );  exit( 101 );
      }
      if( line[j]=='*' ) {        /* moderated group */
        u_moderated[u_groups] = TRUE;
        j++;
      }
      tostring( &groupstxt[u_groups], &line[j], NULL );
      u_groups++;
    } else if( strlook( line, "VOTENAME", 8 ) == 0 ) {
      tostring( &u_votename, &line[j], groupstxt[0] );
    } else if( strlook( line, "VOTEACKSUB", 10 ) == 0 ) {
      tostring( &u_voteacksub, &line[j], u_votename );
    } else if( strlook( line, "VOTEMARK", 8 ) == 0 ) {
      tostring( &u_votemark, &line[j], "%" );
    } else if( strlook( line, "NAMETEXT", 8 ) == 0 ) {
      tostring( &u_nametext, &line[j], NULL );
    } else if( strlook( line, "NEEDNAME", 8 ) == 0 ) {
      u_needname = TRUE;
    } else if( strlook( line, "WANTNAME", 8 ) == 0 ) {
      u_wantname = TRUE;
    } else if( strlook( line, "EXPLICITVOTE", 12 ) == 0 ) {
      u_explicitvote = TRUE;
    } else if( strlook( line, "MAXVOTES", 8 ) == 0 ) {
      u_maxvotes = atoi( &line[j] );
      if( !line[j] || u_maxvotes <= 0 ) {
        fprintf( stderr, "  %s: MAXVOTES needs argument >0 \n", CFGFILE );
        fclose( fin );  exit( 101 );
      }
    } else if( strlook( line, "MEMSORT", 7 ) == 0 ) {
      u_memsort = TRUE;
    } else if( strlook( line, "DOVOTEACK", 9 ) == 0 ) {
      u_dovoteack = TRUE;
    } else if( strlook( line, "RULESMAX", 8 ) == 0 ) {
      u_rulesmax = atoi( &line[j] );
      if( !line[j] || u_rulesmax <= 0 ) {
        fprintf( stderr, "  %s: RULESMAX needs argument >0 \n", CFGFILE );
        fclose( fin );  exit( 101 );
      }
    } else if( strlook( line, "ACKTXTFILE", 10 ) == 0 ) {
      tostring( &u_acktxtfile, &line[j], NULL );
    } else if( strlook( line, "CMDFILE", 7 ) == 0 ) {
      tostring( &u_cmdfile, &line[j], NULL );
    } else if( strlook( line, "RULESFILE", 9 ) == 0 ) {
      tostring( &u_rulesfile, &line[j], NULL );
    } else {
      fprintf( stderr, "  %s unknown: %s\n", CFGFILE, line );
      fclose( fin );  exit( 101 );
    }
  }
  fclose( fin );
}


/* ------------------------------------------------------------------
/* Dump the UseVote configuration
/* ------------------------------------------------------------------
*/
void dump_config()
{
  static char vtype[3][8] = {"CFV", "STV", "STVCFV"};
  static char noyes[2][4] = {"No", "Yes" };
  static char nonetxt[] = "<NULL>";
  int i;

  printf( "\n** UseVote Configuration File: %s\n\n", CFGFILE );
  printf( "  Vote Type: %s\n", vtype[u_votetype] );
  printf( "  Vote Name: %s\n", u_votename );
  printf( "  %d Group(s):\n", u_groups );
  for( i=0; i<u_groups; i++ ) {
    if( u_moderated[i] ) printf( "    %s (moderated)\n", groupstxt[i] );
                    else printf( "    %s\n", groupstxt[i] );
  }
  printf( "  Need voter name: %s\n", noyes[u_needname] );
  printf( "  Want voter name: %s\n", noyes[u_wantname] );
  printf( "  Need explicit 'vote': %s\n", noyes[u_explicitvote] );
  printf( "  Vote acknowledgements: %s\n", noyes[u_dovoteack] );
  printf( "  Vote Ack subject: %s\n", u_voteacksub ? u_voteacksub : nonetxt );
  printf( "  Ballot Marker : %s\n", u_votemark ? u_votemark : nonetxt );
  printf( "  Get name text: %s\n", u_nametext ? u_nametext : nonetxt );
  printf( "\n  Sort in memory: %s\n", noyes[u_memsort] );
  printf( "  Maximum votes: %d\n", u_maxvotes );
  printf( "  Maximum Rules: %d\n", u_rulesmax );
  printf( "  Acknowledgement file: %s\n", u_acktxtfile ? u_acktxtfile : nonetxt );
  printf( "  Send Mail Command file: %s\n", u_cmdfile ? u_cmdfile : nonetxt );
  printf( "  Rules File: %s\n\n", u_rulesfile ? u_rulesfile : nonetxt );
}

/* ------------------------------------------------------------------
/* Called by each piece of UseVote to read the config file
/* ------------------------------------------------------------------
*/
void init_global_config()
{
  init_config();
  read_config();
}

